@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, Optional, Union
 
 import cryptography
 from cryptography.hazmat.primitives import hashes
@@ -23,19 +23,16 @@ from cryptography.hazmat.primitives.serialization import (
     PublicFormat,
 )
 
-from ..const import (  # COSE_KEY_LEN,
+from ..const import (
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT,
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_ES,
+    COSE_ALGORITHMS_HPKE,
     COSE_ALGORITHMS_SIG_OKP,
-    COSE_KEY_LEN,
     COSE_KEY_OPERATION_VALUES,
     COSE_KEY_TYPES,
 )
-from ..cose_key_interface import COSEKeyInterface
 from ..exceptions import EncodeError, VerifyError
-from ..utils import to_cis
 from .asymmetric import AsymmetricKey
-from .symmetric import AESCCMKey, AESGCMKey, ChaCha20Key, HMACKey
 
 
 class OKPKey(AsymmetricKey):
@@ -78,6 +75,8 @@ class OKPKey(AsymmetricKey):
                 self._hash_alg = hashes.SHA256
             elif self._alg in [-26, -28]:
                 self._hash_alg = hashes.SHA512
+            elif self._alg == -1:
+                self._hash_alg = hashes.SHA256 if self._crv == 4 else hashes.SHA512
             else:
                 raise ValueError(f"Unsupported or unknown alg used with X25519/X448: {self._alg}.")
 
@@ -113,6 +112,9 @@ class OKPKey(AsymmetricKey):
                     # public key for key derivation.
                     if self._key_ops:
                         raise ValueError("Public key for ECDHE should not have key_ops.")
+            elif self._alg in COSE_ALGORITHMS_HPKE.values():
+                if not (set(self._key_ops) & set([7, 8])):
+                    raise ValueError("Invalid key_ops for HPKE.")
             else:
                 raise ValueError(f"Unsupported or unknown alg(3) for OKP: {self._alg}.")
         else:
@@ -267,12 +269,7 @@ class OKPKey(AsymmetricKey):
         except cryptography.exceptions.InvalidSignature as err:
             raise VerifyError("Failed to verify.") from err
 
-    def derive_key(
-        self,
-        context: Union[List[Any], Dict[str, Any]],
-        material: bytes = b"",
-        public_key: Optional[COSEKeyInterface] = None,
-    ) -> COSEKeyInterface:
+    def derive_bytes(self, length: int, material: bytes = b"", info: bytes = b"", public_key: Optional[Any] = None) -> bytes:
 
         if self._public_key:
             raise ValueError("Public key cannot be used for key derivation.")
@@ -283,34 +280,14 @@ class OKPKey(AsymmetricKey):
         # if self._alg not in COSE_ALGORITHMS_CKDM_KEY_AGREEMENT.values():
         #     raise ValueError(f"Invalid alg for key derivation: {self._alg}.")
 
-        # Validate context information.
-        if isinstance(context, dict):
-            context = to_cis(context, self._alg)
-        else:
-            self._validate_context(context)
-
         # Derive key.
-        if self._private_key:
-            self._key = self._private_key
-        else:
-            self._key = X25519PrivateKey.generate() if self._crv == 4 else X448PrivateKey.generate()
-        shared_key = self._key.exchange(public_key.key)
-        hkdf = HKDF(
-            algorithm=self._hash_alg(),
-            length=COSE_KEY_LEN[context[0]] // 8,
-            salt=None,
-            info=self._dumps(context),
-        )
-        cose_key = {
-            1: 4,
-            3: context[0],
-            -1: hkdf.derive(shared_key),
-        }
-        if cose_key[3] in [1, 2, 3]:
-            return AESGCMKey(cose_key)
-        if cose_key[3] in [4, 5, 6, 7]:
-            return HMACKey(cose_key)
-        if cose_key[3] in [10, 11, 12, 13, 30, 31, 32, 33]:
-            return AESCCMKey(cose_key)
-        # cose_key[3] == 24:
-        return ChaCha20Key(cose_key)
+        try:
+            if self._private_key:
+                self._key = self._private_key
+            else:
+                self._key = X25519PrivateKey.generate() if self._crv == 4 else X448PrivateKey.generate()
+            shared_key = self._key.exchange(public_key.key)
+            hkdf = HKDF(algorithm=self._hash_alg(), length=length, salt=None, info=info)
+            return hkdf.derive(shared_key)
+        except Exception as err:
+            raise EncodeError("Failed to derive bytes.") from err
