@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Union
 
 from .cbor_processor import CBORProcessor
 from .const import (
@@ -6,7 +6,6 @@ from .const import (
     COSE_KEY_TYPES,
     COSE_NAMED_ALGORITHMS_SUPPORTED,
 )
-from .hpke_cipher_suite import HPKECipherSuite
 
 
 class COSEKeyInterface(CBORProcessor):
@@ -105,6 +104,19 @@ class COSEKeyInterface(CBORProcessor):
         """
         raise NotImplementedError
 
+    @property
+    def crv(self) -> int:
+        """
+        The curve (``crv`` parameter value) of the key.
+        """
+        raise NotImplementedError
+
+    def to_bytes(self) -> bytes:
+        """
+        Serializes the body of the key as a byte string.
+        """
+        raise NotImplementedError
+
     def to_dict(self) -> Dict[int, Any]:
         """
         Returns the CBOR-like structure (Dict[int, Any]) of the COSE key.
@@ -227,39 +239,6 @@ class COSEKeyInterface(CBORProcessor):
         """
         raise NotImplementedError
 
-    def seal(self, suite: HPKECipherSuite, msg: bytes, aad: bytes = b"") -> Tuple[bytes, bytes]:
-        """
-        Encrypts the specified message with HPKE.
-
-        Args:
-            msg (bytes): A message to be encrypted.
-            aad (bytes): Additional authenticated data.
-        Returns:
-            Tuple[bytes, bytes]: The encapsulation key and the ciphertext respectively.
-        Raises:
-            NotImplementedError: Not implemented.
-            ValueError: Invalid arguments.
-            EncodeError: Failed to encrypt the message.
-        """
-        raise NotImplementedError
-
-    def open(self, suite: HPKECipherSuite, enc: bytes, msg: bytes, aad: bytes = b"") -> bytes:
-        """
-        Decrypts the specified message with HPKE.
-
-        Args:
-            enc (bytes): An encapsulated key.
-            msg (bytes): An encrypted message.
-            aad (bytes): Additional authenticated data.
-        Returns:
-            bytes: The byte string of the decrypted data.
-        Raises:
-            NotImplementedError: Not implemented.
-            ValueError: Invalid arguments.
-            DecodeError: Failed to decrypt the message.
-        """
-        raise NotImplementedError
-
     def wrap_key(self, key_to_wrap: bytes) -> bytes:
         """
         Wraps a key.
@@ -290,22 +269,23 @@ class COSEKeyInterface(CBORProcessor):
         """
         raise NotImplementedError
 
-    def derive_key(
+    def derive_bytes(
         self,
-        context: Union[List[Any], Dict[str, Any]],
+        length: int,
         material: bytes = b"",
+        info: bytes = b"",
         public_key: Optional[Any] = None,
-    ) -> Any:
+    ) -> bytes:
         """
-        Derives a key with a key material or key exchange.
+        Derives a byte string with a key material or key exchange.
 
         Args:
-            context (Union[List[Any], Dict[str, Any]]): Context information structure for
-                key derivation functions.
+            length (int): The length of derived byte string.
             material (bytes): A key material as bytes.
+            info (bytes): application supplied information.
             public_key: A public key for key derivation with key exchange.
         Returns:
-            COSEKeyInterface: A COSE key derived.
+            bytes: A derived byte string.
         Raises:
             NotImplementedError: Not implemented.
             ValueError: Invalid arguments.
